# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: aibsmw_messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='aibsmw_messages.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n\x15\x61ibsmw_messages.proto\"V\n\x0emessage_header\x12\x0f\n\x07process\x18\x01 \x02(\t\x12\x0c\n\x04host\x18\x02 \x02(\t\x12\x11\n\ttimestamp\x18\x03 \x02(\x02\x12\x12\n\nmessage_id\x18\x04 \x02(\t\"H\n\x11generic_heartbeat\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nstart_time\x18\x02 \x02(\x02\"\x85\x01\n\x17remote_device_heartbeat\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x13\n\x0b\x64\x65vice_name\x18\x02 \x02(\t\x12\x12\n\nip_address\x18\x04 \x02(\t\x12\x0c\n\x04port\x18\x05 \x02(\x05\x12\x12\n\nstart_time\x18\x06 \x02(\x02\"9\n\x16request_remote_devices\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\"L\n\x0crouter_alive\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x1b\n\x13registered_messages\x18\x02 \x03(\t\"G\n\x13remote_devices_list\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x0f\n\x07\x64\x65vices\x18\x02 \x02(\t\"\x85\x02\n\x16remote_service_request\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12;\n\x0c\x63ommand_type\x18\x02 \x02(\x0e\x32%.remote_service_request.COMMAND_TYPES\x12\x0e\n\x06target\x18\x03 \x02(\t\x12\x0c\n\x04\x61rgs\x18\x04 \x01(\t\x12\x0e\n\x06kwargs\x18\x05 \x01(\t\"_\n\rCOMMAND_TYPES\x12\x0b\n\x07\x43MD_RUN\x10\x00\x12\x0b\n\x07\x43MD_SET\x10\x01\x12\x0b\n\x07\x43MD_GET\x10\x02\x12\x10\n\x0c\x43MD_CALLABLE\x10\x03\x12\x15\n\x11\x43MD_PLATFORM_INFO\x10\x04\"\xc2\x01\n\x14remote_service_reply\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12<\n\x0b\x63\x61ll_result\x18\x02 \x02(\x0e\x32\'.remote_service_reply.CALL_RESULT_TYPES\x12\r\n\x05reply\x18\x03 \x02(\t\"<\n\x11\x43\x41LL_RESULT_TYPES\x12\x11\n\rRESULT_FAILED\x10\x00\x12\x14\n\x10RESULT_PROCESSED\x10\x01\"\xbb\x01\n\x0bpython_info\x12\x14\n\x0c\x62uild_number\x18\x01 \x02(\t\x12\x12\n\nbuild_date\x18\x02 \x02(\t\x12\x10\n\x08\x63ompiler\x18\x03 \x02(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x02(\t\x12\x16\n\x0eimplementation\x18\x05 \x02(\t\x12\x10\n\x08revision\x18\x06 \x02(\t\x12\x0f\n\x07version\x18\x07 \x02(\t\x12\x13\n\x0b\x65xec_prefix\x18\x08 \x02(\t\x12\x10\n\x08is_conda\x18\n \x02(\x08\"\xaa\x01\n\thost_info\x12\x0f\n\x07machine\x18\x01 \x02(\t\x12\x0c\n\x04node\x18\x02 \x02(\t\x12\x10\n\x08platform\x18\x03 \x02(\t\x12\x11\n\tprocessor\x18\x04 \x02(\t\x12\x0f\n\x07release\x18\x05 \x02(\t\x12\x0e\n\x06system\x18\x06 \x02(\t\x12\x0f\n\x07version\x18\x07 \x02(\t\x12\x14\n\x0csys_platform\x18\x08 \x02(\t\x12\x11\n\tbyteorder\x18\t \x02(\t\"|\n\rplatform_info\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x1c\n\x06python\x18\x02 \x02(\x0b\x32\x0c.python_info\x12\x18\n\x04host\x18\x03 \x02(\x0b\x32\n.host_info\x12\x12\n\nstart_time\x18\x04 \x02(\x02\"K\n\x14register_for_message\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nmessage_id\x18\x02 \x02(\t\"M\n\x16\x64\x65register_for_message\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nmessage_id\x18\x02 \x02(\t')
)



_REMOTE_SERVICE_REQUEST_COMMAND_TYPES = _descriptor.EnumDescriptor(
  name='COMMAND_TYPES',
  full_name='remote_service_request.COMMAND_TYPES',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CMD_RUN', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CMD_SET', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CMD_GET', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CMD_CALLABLE', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CMD_PLATFORM_INFO', index=4, number=4,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=700,
  serialized_end=795,
)
_sym_db.RegisterEnumDescriptor(_REMOTE_SERVICE_REQUEST_COMMAND_TYPES)

_REMOTE_SERVICE_REPLY_CALL_RESULT_TYPES = _descriptor.EnumDescriptor(
  name='CALL_RESULT_TYPES',
  full_name='remote_service_reply.CALL_RESULT_TYPES',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RESULT_FAILED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESULT_PROCESSED', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=932,
  serialized_end=992,
)
_sym_db.RegisterEnumDescriptor(_REMOTE_SERVICE_REPLY_CALL_RESULT_TYPES)


_MESSAGE_HEADER = _descriptor.Descriptor(
  name='message_header',
  full_name='message_header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='process', full_name='message_header.process', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host', full_name='message_header.host', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='message_header.timestamp', index=2,
      number=3, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_id', full_name='message_header.message_id', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=25,
  serialized_end=111,
)


_GENERIC_HEARTBEAT = _descriptor.Descriptor(
  name='generic_heartbeat',
  full_name='generic_heartbeat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='generic_heartbeat.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='generic_heartbeat.start_time', index=1,
      number=2, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=113,
  serialized_end=185,
)


_REMOTE_DEVICE_HEARTBEAT = _descriptor.Descriptor(
  name='remote_device_heartbeat',
  full_name='remote_device_heartbeat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='remote_device_heartbeat.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='device_name', full_name='remote_device_heartbeat.device_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ip_address', full_name='remote_device_heartbeat.ip_address', index=2,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='remote_device_heartbeat.port', index=3,
      number=5, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='remote_device_heartbeat.start_time', index=4,
      number=6, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=188,
  serialized_end=321,
)


_REQUEST_REMOTE_DEVICES = _descriptor.Descriptor(
  name='request_remote_devices',
  full_name='request_remote_devices',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='request_remote_devices.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=323,
  serialized_end=380,
)


_ROUTER_ALIVE = _descriptor.Descriptor(
  name='router_alive',
  full_name='router_alive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='router_alive.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='registered_messages', full_name='router_alive.registered_messages', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=382,
  serialized_end=458,
)


_REMOTE_DEVICES_LIST = _descriptor.Descriptor(
  name='remote_devices_list',
  full_name='remote_devices_list',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='remote_devices_list.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='devices', full_name='remote_devices_list.devices', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=460,
  serialized_end=531,
)


_REMOTE_SERVICE_REQUEST = _descriptor.Descriptor(
  name='remote_service_request',
  full_name='remote_service_request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='remote_service_request.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='command_type', full_name='remote_service_request.command_type', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='target', full_name='remote_service_request.target', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='args', full_name='remote_service_request.args', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kwargs', full_name='remote_service_request.kwargs', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _REMOTE_SERVICE_REQUEST_COMMAND_TYPES,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=534,
  serialized_end=795,
)


_REMOTE_SERVICE_REPLY = _descriptor.Descriptor(
  name='remote_service_reply',
  full_name='remote_service_reply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='remote_service_reply.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='call_result', full_name='remote_service_reply.call_result', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reply', full_name='remote_service_reply.reply', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _REMOTE_SERVICE_REPLY_CALL_RESULT_TYPES,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=798,
  serialized_end=992,
)


_PYTHON_INFO = _descriptor.Descriptor(
  name='python_info',
  full_name='python_info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='build_number', full_name='python_info.build_number', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='build_date', full_name='python_info.build_date', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compiler', full_name='python_info.compiler', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='branch', full_name='python_info.branch', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='implementation', full_name='python_info.implementation', index=4,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='python_info.revision', index=5,
      number=6, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='python_info.version', index=6,
      number=7, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exec_prefix', full_name='python_info.exec_prefix', index=7,
      number=8, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_conda', full_name='python_info.is_conda', index=8,
      number=10, type=8, cpp_type=7, label=2,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=995,
  serialized_end=1182,
)


_HOST_INFO = _descriptor.Descriptor(
  name='host_info',
  full_name='host_info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='machine', full_name='host_info.machine', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='node', full_name='host_info.node', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='platform', full_name='host_info.platform', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='processor', full_name='host_info.processor', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='release', full_name='host_info.release', index=4,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='system', full_name='host_info.system', index=5,
      number=6, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='host_info.version', index=6,
      number=7, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sys_platform', full_name='host_info.sys_platform', index=7,
      number=8, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='byteorder', full_name='host_info.byteorder', index=8,
      number=9, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1185,
  serialized_end=1355,
)


_PLATFORM_INFO = _descriptor.Descriptor(
  name='platform_info',
  full_name='platform_info',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='platform_info.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='python', full_name='platform_info.python', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host', full_name='platform_info.host', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='platform_info.start_time', index=3,
      number=4, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1357,
  serialized_end=1481,
)


_REGISTER_FOR_MESSAGE = _descriptor.Descriptor(
  name='register_for_message',
  full_name='register_for_message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='register_for_message.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_id', full_name='register_for_message.message_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1483,
  serialized_end=1558,
)


_DEREGISTER_FOR_MESSAGE = _descriptor.Descriptor(
  name='deregister_for_message',
  full_name='deregister_for_message',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='deregister_for_message.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_id', full_name='deregister_for_message.message_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1560,
  serialized_end=1637,
)

_GENERIC_HEARTBEAT.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REMOTE_DEVICE_HEARTBEAT.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REQUEST_REMOTE_DEVICES.fields_by_name['header'].message_type = _MESSAGE_HEADER
_ROUTER_ALIVE.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REMOTE_DEVICES_LIST.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REMOTE_SERVICE_REQUEST.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REMOTE_SERVICE_REQUEST.fields_by_name['command_type'].enum_type = _REMOTE_SERVICE_REQUEST_COMMAND_TYPES
_REMOTE_SERVICE_REQUEST_COMMAND_TYPES.containing_type = _REMOTE_SERVICE_REQUEST
_REMOTE_SERVICE_REPLY.fields_by_name['header'].message_type = _MESSAGE_HEADER
_REMOTE_SERVICE_REPLY.fields_by_name['call_result'].enum_type = _REMOTE_SERVICE_REPLY_CALL_RESULT_TYPES
_REMOTE_SERVICE_REPLY_CALL_RESULT_TYPES.containing_type = _REMOTE_SERVICE_REPLY
_PLATFORM_INFO.fields_by_name['header'].message_type = _MESSAGE_HEADER
_PLATFORM_INFO.fields_by_name['python'].message_type = _PYTHON_INFO
_PLATFORM_INFO.fields_by_name['host'].message_type = _HOST_INFO
_REGISTER_FOR_MESSAGE.fields_by_name['header'].message_type = _MESSAGE_HEADER
_DEREGISTER_FOR_MESSAGE.fields_by_name['header'].message_type = _MESSAGE_HEADER
DESCRIPTOR.message_types_by_name['message_header'] = _MESSAGE_HEADER
DESCRIPTOR.message_types_by_name['generic_heartbeat'] = _GENERIC_HEARTBEAT
DESCRIPTOR.message_types_by_name['remote_device_heartbeat'] = _REMOTE_DEVICE_HEARTBEAT
DESCRIPTOR.message_types_by_name['request_remote_devices'] = _REQUEST_REMOTE_DEVICES
DESCRIPTOR.message_types_by_name['router_alive'] = _ROUTER_ALIVE
DESCRIPTOR.message_types_by_name['remote_devices_list'] = _REMOTE_DEVICES_LIST
DESCRIPTOR.message_types_by_name['remote_service_request'] = _REMOTE_SERVICE_REQUEST
DESCRIPTOR.message_types_by_name['remote_service_reply'] = _REMOTE_SERVICE_REPLY
DESCRIPTOR.message_types_by_name['python_info'] = _PYTHON_INFO
DESCRIPTOR.message_types_by_name['host_info'] = _HOST_INFO
DESCRIPTOR.message_types_by_name['platform_info'] = _PLATFORM_INFO
DESCRIPTOR.message_types_by_name['register_for_message'] = _REGISTER_FOR_MESSAGE
DESCRIPTOR.message_types_by_name['deregister_for_message'] = _DEREGISTER_FOR_MESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

message_header = _reflection.GeneratedProtocolMessageType('message_header', (_message.Message,), dict(
  DESCRIPTOR = _MESSAGE_HEADER,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:message_header)
  ))
_sym_db.RegisterMessage(message_header)

generic_heartbeat = _reflection.GeneratedProtocolMessageType('generic_heartbeat', (_message.Message,), dict(
  DESCRIPTOR = _GENERIC_HEARTBEAT,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:generic_heartbeat)
  ))
_sym_db.RegisterMessage(generic_heartbeat)

remote_device_heartbeat = _reflection.GeneratedProtocolMessageType('remote_device_heartbeat', (_message.Message,), dict(
  DESCRIPTOR = _REMOTE_DEVICE_HEARTBEAT,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:remote_device_heartbeat)
  ))
_sym_db.RegisterMessage(remote_device_heartbeat)

request_remote_devices = _reflection.GeneratedProtocolMessageType('request_remote_devices', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST_REMOTE_DEVICES,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:request_remote_devices)
  ))
_sym_db.RegisterMessage(request_remote_devices)

router_alive = _reflection.GeneratedProtocolMessageType('router_alive', (_message.Message,), dict(
  DESCRIPTOR = _ROUTER_ALIVE,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:router_alive)
  ))
_sym_db.RegisterMessage(router_alive)

remote_devices_list = _reflection.GeneratedProtocolMessageType('remote_devices_list', (_message.Message,), dict(
  DESCRIPTOR = _REMOTE_DEVICES_LIST,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:remote_devices_list)
  ))
_sym_db.RegisterMessage(remote_devices_list)

remote_service_request = _reflection.GeneratedProtocolMessageType('remote_service_request', (_message.Message,), dict(
  DESCRIPTOR = _REMOTE_SERVICE_REQUEST,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:remote_service_request)
  ))
_sym_db.RegisterMessage(remote_service_request)

remote_service_reply = _reflection.GeneratedProtocolMessageType('remote_service_reply', (_message.Message,), dict(
  DESCRIPTOR = _REMOTE_SERVICE_REPLY,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:remote_service_reply)
  ))
_sym_db.RegisterMessage(remote_service_reply)

python_info = _reflection.GeneratedProtocolMessageType('python_info', (_message.Message,), dict(
  DESCRIPTOR = _PYTHON_INFO,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:python_info)
  ))
_sym_db.RegisterMessage(python_info)

host_info = _reflection.GeneratedProtocolMessageType('host_info', (_message.Message,), dict(
  DESCRIPTOR = _HOST_INFO,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:host_info)
  ))
_sym_db.RegisterMessage(host_info)

platform_info = _reflection.GeneratedProtocolMessageType('platform_info', (_message.Message,), dict(
  DESCRIPTOR = _PLATFORM_INFO,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:platform_info)
  ))
_sym_db.RegisterMessage(platform_info)

register_for_message = _reflection.GeneratedProtocolMessageType('register_for_message', (_message.Message,), dict(
  DESCRIPTOR = _REGISTER_FOR_MESSAGE,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:register_for_message)
  ))
_sym_db.RegisterMessage(register_for_message)

deregister_for_message = _reflection.GeneratedProtocolMessageType('deregister_for_message', (_message.Message,), dict(
  DESCRIPTOR = _DEREGISTER_FOR_MESSAGE,
  __module__ = 'aibsmw_messages_pb2'
  # @@protoc_insertion_point(class_scope:deregister_for_message)
  ))
_sym_db.RegisterMessage(deregister_for_message)


# @@protoc_insertion_point(module_scope)
