# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: middleware_messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import aibsmw_messages_pb2 as aibsmw__messages__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='middleware_messages.proto',
  package='',
  syntax='proto2',
  serialized_pb=_b('\n\x19middleware_messages.proto\x1a\x15\x61ibsmw_messages.proto\"=\n\x08workflow\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x10\n\x08workflow\x18\x02 \x02(\t\"3\n\x10workflow_request\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\"H\n\rworkflow_list\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x16\n\x0eworkflow_files\x18\x02 \x03(\t\"f\n\rworkflow_load\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x19\n\x11workflow_filename\x18\x02 \x02(\t\x12\x19\n\ndummy_mode\x18\x03 \x02(\x08:\x05\x66\x61lse\"8\n\x15workflow_list_request\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\"L\n\x15workflow_revert_state\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nstate_name\x18\x02 \x02(\t\"`\n\x13transition_to_state\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nstate_name\x18\x02 \x02(\t\x12\x14\n\x0cstate_object\x18\x03 \x02(\t\"b\n\x15transition_to_cleanup\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nstate_name\x18\x02 \x02(\t\x12\x14\n\x0cstate_object\x18\x03 \x02(\t\"\xc6\x01\n\rstate_changed\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x12\n\nstate_name\x18\x02 \x02(\t\x12+\n\x06result\x18\x03 \x02(\x0e\x32\x1b.state_changed.result_types\x12\x16\n\x0estatus_message\x18\x04 \x02(\t\x12\x14\n\x0cstate_object\x18\x05 \x02(\t\"%\n\x0cresult_types\x12\x0b\n\x07SUCCESS\x10\x00\x12\x08\n\x04\x46\x41IL\x10\x01\">\n\nstate_busy\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x0f\n\x07message\x18\x02 \x02(\t\"?\n\x0bstate_ready\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x0f\n\x07message\x18\x02 \x02(\t\"7\n\x03log\x12\x1f\n\x06header\x18\x01 \x02(\x0b\x32\x0f.message_header\x12\x0f\n\x07message\x18\x02 \x02(\t')
  ,
  dependencies=[aibsmw__messages__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_STATE_CHANGED_RESULT_TYPES = _descriptor.EnumDescriptor(
  name='result_types',
  full_name='state_changed.result_types',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUCCESS', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAIL', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=842,
  serialized_end=879,
)
_sym_db.RegisterEnumDescriptor(_STATE_CHANGED_RESULT_TYPES)


_WORKFLOW = _descriptor.Descriptor(
  name='workflow',
  full_name='workflow',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workflow', full_name='workflow.workflow', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=52,
  serialized_end=113,
)


_WORKFLOW_REQUEST = _descriptor.Descriptor(
  name='workflow_request',
  full_name='workflow_request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow_request.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=115,
  serialized_end=166,
)


_WORKFLOW_LIST = _descriptor.Descriptor(
  name='workflow_list',
  full_name='workflow_list',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow_list.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workflow_files', full_name='workflow_list.workflow_files', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=168,
  serialized_end=240,
)


_WORKFLOW_LOAD = _descriptor.Descriptor(
  name='workflow_load',
  full_name='workflow_load',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow_load.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workflow_filename', full_name='workflow_load.workflow_filename', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dummy_mode', full_name='workflow_load.dummy_mode', index=2,
      number=3, type=8, cpp_type=7, label=2,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=242,
  serialized_end=344,
)


_WORKFLOW_LIST_REQUEST = _descriptor.Descriptor(
  name='workflow_list_request',
  full_name='workflow_list_request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow_list_request.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=346,
  serialized_end=402,
)


_WORKFLOW_REVERT_STATE = _descriptor.Descriptor(
  name='workflow_revert_state',
  full_name='workflow_revert_state',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='workflow_revert_state.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_name', full_name='workflow_revert_state.state_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=404,
  serialized_end=480,
)


_TRANSITION_TO_STATE = _descriptor.Descriptor(
  name='transition_to_state',
  full_name='transition_to_state',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='transition_to_state.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_name', full_name='transition_to_state.state_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_object', full_name='transition_to_state.state_object', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=482,
  serialized_end=578,
)


_TRANSITION_TO_CLEANUP = _descriptor.Descriptor(
  name='transition_to_cleanup',
  full_name='transition_to_cleanup',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='transition_to_cleanup.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_name', full_name='transition_to_cleanup.state_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_object', full_name='transition_to_cleanup.state_object', index=2,
      number=3, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=580,
  serialized_end=678,
)


_STATE_CHANGED = _descriptor.Descriptor(
  name='state_changed',
  full_name='state_changed',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='state_changed.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_name', full_name='state_changed.state_name', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='result', full_name='state_changed.result', index=2,
      number=3, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='status_message', full_name='state_changed.status_message', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state_object', full_name='state_changed.state_object', index=4,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _STATE_CHANGED_RESULT_TYPES,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=681,
  serialized_end=879,
)


_STATE_BUSY = _descriptor.Descriptor(
  name='state_busy',
  full_name='state_busy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='state_busy.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='state_busy.message', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=881,
  serialized_end=943,
)


_STATE_READY = _descriptor.Descriptor(
  name='state_ready',
  full_name='state_ready',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='state_ready.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='state_ready.message', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=945,
  serialized_end=1008,
)


_LOG = _descriptor.Descriptor(
  name='log',
  full_name='log',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='log.header', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='log.message', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1010,
  serialized_end=1065,
)

_WORKFLOW.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_WORKFLOW_REQUEST.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_WORKFLOW_LIST.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_WORKFLOW_LOAD.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_WORKFLOW_LIST_REQUEST.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_WORKFLOW_REVERT_STATE.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_TRANSITION_TO_STATE.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_TRANSITION_TO_CLEANUP.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_STATE_CHANGED.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_STATE_CHANGED.fields_by_name['result'].enum_type = _STATE_CHANGED_RESULT_TYPES
_STATE_CHANGED_RESULT_TYPES.containing_type = _STATE_CHANGED
_STATE_BUSY.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_STATE_READY.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
_LOG.fields_by_name['header'].message_type = aibsmw__messages__pb2._MESSAGE_HEADER
DESCRIPTOR.message_types_by_name['workflow'] = _WORKFLOW
DESCRIPTOR.message_types_by_name['workflow_request'] = _WORKFLOW_REQUEST
DESCRIPTOR.message_types_by_name['workflow_list'] = _WORKFLOW_LIST
DESCRIPTOR.message_types_by_name['workflow_load'] = _WORKFLOW_LOAD
DESCRIPTOR.message_types_by_name['workflow_list_request'] = _WORKFLOW_LIST_REQUEST
DESCRIPTOR.message_types_by_name['workflow_revert_state'] = _WORKFLOW_REVERT_STATE
DESCRIPTOR.message_types_by_name['transition_to_state'] = _TRANSITION_TO_STATE
DESCRIPTOR.message_types_by_name['transition_to_cleanup'] = _TRANSITION_TO_CLEANUP
DESCRIPTOR.message_types_by_name['state_changed'] = _STATE_CHANGED
DESCRIPTOR.message_types_by_name['state_busy'] = _STATE_BUSY
DESCRIPTOR.message_types_by_name['state_ready'] = _STATE_READY
DESCRIPTOR.message_types_by_name['log'] = _LOG

workflow = _reflection.GeneratedProtocolMessageType('workflow', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow)
  ))
_sym_db.RegisterMessage(workflow)

workflow_request = _reflection.GeneratedProtocolMessageType('workflow_request', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW_REQUEST,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow_request)
  ))
_sym_db.RegisterMessage(workflow_request)

workflow_list = _reflection.GeneratedProtocolMessageType('workflow_list', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW_LIST,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow_list)
  ))
_sym_db.RegisterMessage(workflow_list)

workflow_load = _reflection.GeneratedProtocolMessageType('workflow_load', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW_LOAD,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow_load)
  ))
_sym_db.RegisterMessage(workflow_load)

workflow_list_request = _reflection.GeneratedProtocolMessageType('workflow_list_request', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW_LIST_REQUEST,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow_list_request)
  ))
_sym_db.RegisterMessage(workflow_list_request)

workflow_revert_state = _reflection.GeneratedProtocolMessageType('workflow_revert_state', (_message.Message,), dict(
  DESCRIPTOR = _WORKFLOW_REVERT_STATE,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:workflow_revert_state)
  ))
_sym_db.RegisterMessage(workflow_revert_state)

transition_to_state = _reflection.GeneratedProtocolMessageType('transition_to_state', (_message.Message,), dict(
  DESCRIPTOR = _TRANSITION_TO_STATE,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:transition_to_state)
  ))
_sym_db.RegisterMessage(transition_to_state)

transition_to_cleanup = _reflection.GeneratedProtocolMessageType('transition_to_cleanup', (_message.Message,), dict(
  DESCRIPTOR = _TRANSITION_TO_CLEANUP,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:transition_to_cleanup)
  ))
_sym_db.RegisterMessage(transition_to_cleanup)

state_changed = _reflection.GeneratedProtocolMessageType('state_changed', (_message.Message,), dict(
  DESCRIPTOR = _STATE_CHANGED,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:state_changed)
  ))
_sym_db.RegisterMessage(state_changed)

state_busy = _reflection.GeneratedProtocolMessageType('state_busy', (_message.Message,), dict(
  DESCRIPTOR = _STATE_BUSY,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:state_busy)
  ))
_sym_db.RegisterMessage(state_busy)

state_ready = _reflection.GeneratedProtocolMessageType('state_ready', (_message.Message,), dict(
  DESCRIPTOR = _STATE_READY,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:state_ready)
  ))
_sym_db.RegisterMessage(state_ready)

log = _reflection.GeneratedProtocolMessageType('log', (_message.Message,), dict(
  DESCRIPTOR = _LOG,
  __module__ = 'middleware_messages_pb2'
  # @@protoc_insertion_point(class_scope:log)
  ))
_sym_db.RegisterMessage(log)


# @@protoc_insertion_point(module_scope)
