from np_workflows.workflows.shared import npxc

import np_logging

# def initialize_enter(state_globals) -> None: ... 
# def initialize_input(state_globals) -> None: ...
# def initialize_exit(state_globals) -> None: ...
def msg_enter(state_globals) -> None: 
    import pdb; pdb.set_trace()
    state_globals["external"]["transition_result"] = False # T/F just sets text at bottom green/red - switches to T in next state
    state_globals["external"]["status_message"] = 'status_message at bottom'
    state_globals["external"]["msg_text"] = 'msg_text in main widget'
    state_globals['external']['alert'] = True # T adds warning icon 
    # state_globals["external"]["next_state"] = 'warn' # sets next state by name
# def msg_input(state_globals) -> None: ... 
# def msg_exit(state_globals) -> None: ...
# def warn_enter(state_globals) -> None: ...
# def warn_input(state_globals) -> None: ...
# def workflow_complete_enter(state_globals) -> None: ...
# def workflow_complete_input(state_globals) -> None: ...
# def workflow_complete_exit(state_globals) -> None: ...